/**
 * EasyBeans
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MessageDrivenBeanContext.java 5644 2010-10-19 12:40:11Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.messagedrivenbean;

import java.util.concurrent.atomic.AtomicInteger;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;

/**
 * Example of Message Driven Bean that can use the MDB context.<br />
 * This is an JMS MDB, implementing the MessageListener interface.
 * @author Florent Benoit
 */
@MessageDriven(messageListenerInterface = MessageListener.class,
        activationConfig = {
        @ActivationConfigProperty(propertyName = "destination", propertyValue = "SampleRollbackQueue"),
        @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue")})
public class MessageDrivenBeanContext {

    /**
     * Bean's context.
     */
    @Resource
    private MessageDrivenContext context;

    /**
     * Counter.
     */
    private static AtomicInteger count = new AtomicInteger(0);


    /**
     * Delivery that can fails.
     */
    public static final int DELIVERY_FAIL = 5;

    /**
     * Passes a message to the listener.
     * @param message - the message passed to the listener
     */
    public void onMessage(final Message message) {

        // sets the transaction as rollback only if count < 5
        // This will redeliver the message
        if (count.get() < DELIVERY_FAIL) {
            this.context.setRollbackOnly();
            System.out.println("Sets rollback only mode. Counter = " + count.incrementAndGet());
        } else {
            count.set(0);
            System.out.println("got " + DELIVERY_FAIL + " rollbacks, now accepting the message");
        }

    }

}
